#include "project-defs.h"
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "txt-abstract-controller.h"
#include "txt-module.h"

void txtInitialise(TxtModule *module) {
	commLinkInitialise(&module->link);
	module->controller->initialise(module);
}

void txtClear(TxtModule *module) {
	module->controller->clear(module);
}

static void printStringAt(TxtModule *module, uint8_t column, uint8_t row, char *s) {
	// Les chaînes de caractères latins peuvent commencer à n'importe quelle
	// position pourvu que leur longueur soit un nombre pair.
	// Les caractères chinois sur 16 bits peuvent commencer seulement à une
	// position paire.
	// Par souci de simplicité, on commencera toujours à une position paire.
	column &= ~1;

	// On positionne le curseur à la position voulue.
	module->controller->setCursorPosition(module, column, row);

	// On s'assure que la longueur de la chaîne soit paire en ajoutant au
	// besoin un espace à la fin.
	uint8_t l = strlen(s);

	if (l & 1) {
		s[l] = ' ';
		s[++l] = '\0';
	}

	// On tronque la chaîne si elle déborde de la ligne d'affichage.
	uint8_t lMax = module->display.width - column;
	lMax &= ~1;

	if (l > lMax) {
		s[lMax] = '\0';
		l = lMax;
	}

	// On peut maintenant envoyer les caractères au contrôleur.
	for (uint8_t i = 0; i < l; i++) {
		module->controller->writeByte(module, s[i]);
	}
}

static char buffer[81];

void txtPrintAt(TxtModule *module, int column, int row, const char *fmt, ...) {
	va_list vaList;

	va_start(vaList, fmt);
	vsnprintf(buffer, sizeof(buffer), fmt, vaList);
	va_end(vaList);

	printStringAt(module, column, row, buffer);
}
