#ifndef _TXT_MODULE_H
#define _TXT_MODULE_H

/**
 * @file txt-module.h
 *
 * Définitions relatives au module d'affichage texte dans son ensemble
 * et fonctions de haut niveau pour l'utiliser facilement.
 */

#include "txt-display.h"
#include "comm-link-abstract.h"

struct TxtController;

/**
 * Cette structure contient la description de tous les composants d'un
 * module d'affichage en mode texte. Pour pouvoir utiliser un module,
 * il faut initialiser chaque membre de cette structure avec les données
 * de configuration et les blocs de fonctions correspondant aux éléments
 * de la data sheet du module.
 */
typedef struct {
	// Caractéristiques de l'écran
	const TxtDisplay display;
	// Contrôleur
	const struct TxtController *controller;
	// Interface de communication
	const CommLink link;
} TxtModule;

/**
 * Initialisation du module.
 *
 * @param module Pointeur sur la structure de description du module LCD.
 */
void txtInitialise(TxtModule *module);
/**
 * Efface complètement l'affichage
 *
 * @param module Pointeur sur la structure de description du module LCD.
 */
void txtClear(TxtModule *module);
/**
 * Equivalent de printf() avec positionnement de la chaîne sur l'écran.
 *
 * @param module Pointeur sur la structure de description du module LCD.
 * @param column Colonne de départ (commence à 0, incrémente vers la droite).
 * @param row Ligne de départ (commence à 0, incrémente vers le bas).
 * @param fmt Chaîne de formatage identique à celle de printf().
 * Les éventuels arguments suivent, exactement comme avec printf().
 */
void txtPrintAt(TxtModule *module, int column, int row, const char *fmt, ...);

#endif // _TXT_MODULE_H
